'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VSBrowser = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const compare_versions_1 = require("compare-versions");
const monaco_page_objects_1 = require("monaco-page-objects");
const chrome_1 = require("selenium-webdriver/chrome");
const vscode_extension_tester_locators_1 = require("vscode-extension-tester-locators");
const codeUtil_1 = require("./util/codeUtil");
const extester_1 = require("./extester");
const driverUtil_1 = require("./util/driverUtil");
class VSBrowser {
    constructor(codeVersion, releaseType, customSettings = {}, logLevel = monaco_page_objects_1.logging.Level.INFO) {
        this.storagePath = process.env.TEST_RESOURCES ? process.env.TEST_RESOURCES : path.resolve(extester_1.DEFAULT_STORAGE_FOLDER);
        this.extensionsFolder = process.env.EXTENSIONS_FOLDER ? process.env.EXTENSIONS_FOLDER : undefined;
        this.customSettings = customSettings;
        this.codeVersion = codeVersion;
        this.releaseType = releaseType;
        this.logLevel = logLevel;
        VSBrowser._instance = this;
    }
    ;
    /**
     * Starts the vscode browser from a given path
     * @param codePath path to code binary
     */
    start(codePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const userSettings = path.join(this.storagePath, 'settings', 'User');
            if (fs.existsSync(userSettings)) {
                fs.removeSync(path.join(this.storagePath, 'settings'));
            }
            let defaultSettings = {
                "workbench.editor.enablePreview": false,
                "workbench.startupEditor": "none",
                "window.titleBarStyle": "custom",
                "window.commandCenter": false,
                "window.dialogStyle": "custom",
                "window.restoreFullscreen": true,
                "window.newWindowDimensions": "maximized",
                "security.workspace.trust.enabled": false,
                "files.simpleDialog.enable": true,
                "terminal.integrated.copyOnSelection": true
            };
            if (Object.keys(this.customSettings).length > 0) {
                console.log('Detected user defined code settings');
                defaultSettings = Object.assign(Object.assign({}, defaultSettings), this.customSettings);
            }
            fs.mkdirpSync(path.join(userSettings, 'globalStorage'));
            yield fs.remove(path.join(this.storagePath, 'screenshots'));
            fs.writeJSONSync(path.join(userSettings, 'settings.json'), defaultSettings);
            console.log(`Writing code settings to ${path.join(userSettings, 'settings.json')}`);
            const args = ['--no-sandbox', '--disable-dev-shm-usage', `--user-data-dir=${path.join(this.storagePath, 'settings')}`];
            if (this.extensionsFolder) {
                args.push(`--extensions-dir=${this.extensionsFolder}`);
            }
            if ((0, compare_versions_1.compareVersions)(this.codeVersion, '1.39.0') < 0) {
                if (process.platform === 'win32') {
                    fs.copyFileSync(path.resolve(__dirname, '..', '..', 'resources', 'state.vscdb'), path.join(userSettings, 'globalStorage', 'state.vscdb'));
                }
                args.push(`--extensionDevelopmentPath=${process.cwd()}`);
            }
            let options = new chrome_1.Options().setChromeBinaryPath(codePath).addArguments(...args);
            options['options_'].windowTypes = ['webview'];
            options = options;
            const prefs = new monaco_page_objects_1.logging.Preferences();
            prefs.setLevel(monaco_page_objects_1.logging.Type.DRIVER, this.logLevel);
            options.setLoggingPrefs(prefs);
            const driverBinary = process.platform === 'win32' ? 'chromedriver.exe' : 'chromedriver';
            let chromeDriverBinaryPath = path.join(this.storagePath, driverBinary);
            if (this.codeVersion >= '1.86.0') {
                chromeDriverBinaryPath = path.join(this.storagePath, `chromedriver-${driverUtil_1.DriverUtil.getChromeDriverPlatform()}`, driverBinary);
            }
            console.log('Launching browser...');
            this._driver = yield new monaco_page_objects_1.Builder()
                .setChromeService(new chrome_1.ServiceBuilder(chromeDriverBinaryPath))
                .forBrowser(monaco_page_objects_1.Browser.CHROME)
                .setChromeOptions(options)
                .build();
            VSBrowser._instance = this;
            (0, monaco_page_objects_1.initPageObjects)(this.codeVersion, VSBrowser.baseVersion, (0, vscode_extension_tester_locators_1.getLocatorsPath)(), this._driver, VSBrowser.browserName);
            return this;
        });
    }
    /**
     * Returns a reference to the underlying instance of Webdriver
     */
    get driver() {
        return this._driver;
    }
    /**
     * Returns the vscode version as string
     */
    get version() {
        return this.codeVersion;
    }
    /**
     * Returns an instance of VSBrowser
     */
    static get instance() {
        return VSBrowser._instance;
    }
    /**
     * Waits until parts of the workbench are loaded
     */
    waitForWorkbench(timeout = 30000) {
        return __awaiter(this, void 0, void 0, function* () {
            // Workaround/patch for https://github.com/redhat-developer/vscode-extension-tester/issues/466
            try {
                yield this._driver.wait(monaco_page_objects_1.until.elementLocated(monaco_page_objects_1.By.className('monaco-workbench')), timeout, `Workbench was not loaded properly after ${timeout} ms.`);
            }
            catch (err) {
                if (err.name === 'WebDriverError') {
                    yield new Promise(res => setTimeout(res, 3000));
                }
                else {
                    throw err;
                }
            }
        });
    }
    /**
     * Terminates the webdriver/browser
     */
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            const entries = yield this._driver.manage().logs().get(monaco_page_objects_1.logging.Type.DRIVER);
            const logFile = path.join(this.storagePath, 'test.log');
            const stream = fs.createWriteStream(logFile, { flags: 'w' });
            entries.forEach(entry => {
                stream.write(`[${new Date(entry.timestamp).toLocaleTimeString()}][${entry.level.name}] ${entry.message}`);
            });
            stream.end();
            console.log('Shutting down the browser');
            yield this._driver.quit();
        });
    }
    /**
     * Take a screenshot of the browser
     * @param name file name of the screenshot without extension
     */
    takeScreenshot(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this._driver.takeScreenshot();
            const dir = path.join(this.storagePath, 'screenshots');
            fs.mkdirpSync(dir);
            fs.writeFileSync(path.join(dir, `${name}.png`), data, 'base64');
        });
    }
    /**
     * Get a screenshots folder path
     * @returns string path to the screenshots folder
     */
    getScreenshotsDir() {
        return path.join(this.storagePath, 'screenshots');
    }
    /**
     * Open folder(s) or file(s) in the current instance of vscode.
     *
     * @param paths path(s) of folder(s)/files(s) to open as varargs
     * @returns Promise resolving when all selected resources are opened and the workbench reloads
     */
    openResources(...paths) {
        return __awaiter(this, void 0, void 0, function* () {
            if (paths.length === 0) {
                return;
            }
            const code = new codeUtil_1.CodeUtil(this.storagePath, this.releaseType, this.extensionsFolder);
            code.open(...paths);
            yield new Promise(res => setTimeout(res, 3000));
            yield this.waitForWorkbench();
        });
    }
}
exports.VSBrowser = VSBrowser;
VSBrowser.baseVersion = '1.37.0';
VSBrowser.browserName = 'vscode';
//# sourceMappingURL=browser.js.map