#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const extester_1 = require("./extester");
const codeUtil_1 = require("./util/codeUtil");
const package_json_1 = __importDefault(require("../package.json"));
commander_1.program.version(package_json_1.default.version)
    .description('UI Test Runner for VS Code Extension');
commander_1.program.command('get-vscode')
    .description('Download VS Code for testing')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-c, --code_version <version>', 'Version of VS Code to download, use `min`/`max` to download the oldest/latest VS Code supported by ExTester')
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .action(withErrors((cmd) => __awaiter(void 0, void 0, void 0, function* () {
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type));
    yield extest.downloadCode(cmd.code_version);
})));
commander_1.program.command('get-chromedriver')
    .description('Download ChromeDriver binary')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-c, --code_version <version>', 'Version of VS Code you want to run with the ChromeDriver, use `min`/`max` to download the oldest/latest VS Code supported by ExTester')
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .action(withErrors((cmd) => __awaiter(void 0, void 0, void 0, function* () {
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type));
    yield extest.downloadChromeDriver(cmd.code_version);
})));
commander_1.program.command('install-vsix')
    .description('Install extension from vsix file into test instance of VS Code')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-e, --extensions_dir <extensions_directory>', 'VS Code will use this directory for managing extensions')
    .option('-f, --vsix_file <file>', 'path/URL to vsix file containing the extension')
    .option('-y, --yarn', 'Use yarn to build the extension via vsce instead of npm', false)
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .option('-i, --install_dependencies', 'Automatically install extensions your extension depends on', false)
    .action(withErrors((cmd) => __awaiter(void 0, void 0, void 0, function* () {
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type), cmd.extensions_dir);
    yield extest.installVsix({ vsixFile: cmd.vsix_file, useYarn: cmd.yarn, installDependencies: cmd.install_dependencies });
})));
commander_1.program.command('install-from-marketplace <id> [ids...]')
    .description('Install extension from marketplace with given <id> into test instance of VS Code')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-e, --extensions_dir <extensions_directory>', 'VS Code will use this directory for managing extensions')
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .action(withErrors((id, ids, cmd) => __awaiter(void 0, void 0, void 0, function* () {
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type), cmd.extensions_dir);
    yield extest.installFromMarketplace(id);
    if (ids && ids.length > 0) {
        for (const idx of ids) {
            yield extest.installFromMarketplace(idx);
        }
    }
})));
commander_1.program.command('setup-tests')
    .description('Set up all necessary requirements for tests to run')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-e, --extensions_dir <extensions_directory>', 'VS Code will use this directory for managing extensions')
    .option('-c, --code_version <version>', 'Version of VS Code to download, use `min`/`max` to download the oldest/latest VS Code supported by ExTester')
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .option('-y, --yarn', 'Use yarn to build the extension via vsce instead of npm', false)
    .option('-i, --install_dependencies', 'Automatically install extensions your extension depends on', false)
    .action(withErrors((cmd) => __awaiter(void 0, void 0, void 0, function* () {
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type), cmd.extensions_dir);
    yield extest.setupRequirements({ vscodeVersion: cmd.code_version, useYarn: cmd.yarn, installDependencies: cmd.install_dependencies });
})));
commander_1.program.command('run-tests <testFiles...>')
    .description('Run the test files specified by glob pattern(s)')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-e, --extensions_dir <extensions_directory>', 'VS Code will use this directory for managing extensions')
    .option('-c, --code_version <version>', 'Version of VS Code to be used, use `min`/`max` to download the oldest/latest VS Code supported by ExTester')
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .option('-o, --code_settings <settings.json>', 'Path to custom settings for VS Code json file')
    .option('-u, --uninstall_extension', 'Uninstall the extension after the test run', false)
    .option('-m, --mocha_config <mocharc.js>', 'Path to Mocha configuration file')
    .option('-l, --log_level <level>', 'Log messages from webdriver with a given level', 'Info')
    .option('-f, --offline', 'Attempt to run without internet connection, make sure to have all requirements downloaded', false)
    .option('-r, --open_resource <resources...>', 'Open resources in VS Code. Multiple files and folders can be specified.')
    .action(withErrors((testFiles, cmd) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type), cmd.extensions_dir);
    yield extest.runTests(testFiles, { vscodeVersion: cmd.code_version, settings: cmd.code_settings, cleanup: cmd.uninstall_extension, config: cmd.mocha_config, logLevel: cmd.log_level, offline: cmd.offline, resources: (_a = cmd.open_resource) !== null && _a !== void 0 ? _a : [] });
})));
commander_1.program.command('setup-and-run <testFiles...>')
    .description('Perform all setup and run tests specified by glob pattern(s)')
    .option('-s, --storage <storage>', 'Use this folder for all test resources')
    .option('-e, --extensions_dir <extensions_directory>', 'VS Code will use this directory for managing extensions')
    .option('-c, --code_version <version>', 'Version of VS Code to download, use `min`/`max` to download the oldest/latest VS Code supported by ExTester')
    .option('-t, --type <type>', 'Type of VS Code release (stable/insider)')
    .option('-o, --code_settings <settings.json>', 'Path to custom settings for VS Code json file')
    .option('-y, --yarn', 'Use yarn to build the extension via vsce instead of npm', false)
    .option('-u, --uninstall_extension', 'Uninstall the extension after the test run', false)
    .option('-m, --mocha_config <mocharc.js>', 'Path to Mocha configuration file')
    .option('-i, --install_dependencies', 'Automatically install extensions your extension depends on', false)
    .option('-l, --log_level <level>', 'Log messages from webdriver with a given level', 'Info')
    .option('-f, --offline', 'Attempt to run without internet connection, make sure to have all requirements downloaded', false)
    .option('-r, --open_resource <resources...>', 'Open resources in VS Code. Multiple files and folders can be specified.')
    .action(withErrors((testFiles, cmd) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    const extest = new extester_1.ExTester(cmd.storage, codeStream(cmd.type), cmd.extensions_dir);
    yield extest.setupAndRunTests(testFiles, cmd.code_version, { useYarn: cmd.yarn, installDependencies: cmd.install_dependencies }, { settings: cmd.code_settings, cleanup: cmd.uninstall_extension, config: cmd.mocha_config, logLevel: cmd.log_level, resources: (_b = cmd.open_resource) !== null && _b !== void 0 ? _b : [] });
})));
commander_1.program.parse(process.argv);
function withErrors(command) {
    return (...args) => __awaiter(this, void 0, void 0, function* () {
        try {
            yield command(...args);
        }
        catch (err) {
            if (err instanceof Error) {
                console.log(err.stack);
            }
            else {
                console.log(err);
            }
            process.exitCode = 1;
        }
    });
}
function codeStream(stream) {
    const envType = process.env.CODE_TYPE;
    let type = stream;
    if (!type && envType) {
        type = envType;
    }
    if (type && type.toLowerCase() === 'insider') {
        return codeUtil_1.ReleaseQuality.Insider;
    }
    return codeUtil_1.ReleaseQuality.Stable;
}
//# sourceMappingURL=cli.js.map